'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jestMatcherUtils = require('jest-matcher-utils');
var utilities = require('./utilities.js');

function toContainReactText(node, text) {
  utilities.assertIsNode(node, {
    expectation: 'toContainReactText',
    isNot: this.isNot
  });
  const nodeText = node.text();
  const matchIndex = nodeText.indexOf(text);
  const pass = matchIndex >= 0;
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toContainReactText', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to contain text:\n  ${jestMatcherUtils.printExpected(text)}\n` + `But it did:\n  ${printReceivedWithHighlight(nodeText, matchIndex, text.length)}\n` : () => `${jestMatcherUtils.matcherHint('.not.toContainReactText', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `With text content:\n  ${jestMatcherUtils.printReceived(nodeText)}\n` + `To contain string:\n  ${jestMatcherUtils.printExpected(text)}\n`;
  return {
    pass,
    message
  };
}
function toContainReactHtml(node, text) {
  utilities.assertIsNode(node, {
    expectation: 'toContainReactHtml',
    isNot: this.isNot
  });
  const nodeHtml = node.html();
  const matchIndex = nodeHtml.indexOf(text);
  const pass = matchIndex >= 0;
  const message = pass ? () => `${jestMatcherUtils.matcherHint('.not.toContainReactHtml', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `Not to contain HTML:\n  ${jestMatcherUtils.printExpected(text)}\n` + `But it did:\n  ${printReceivedWithHighlight(nodeHtml, matchIndex, text.length)}\n` : () => `${jestMatcherUtils.matcherHint('.not.toContainReactHtml', node.toString())}\n\n` + `Expected the React element:\n  ${jestMatcherUtils.RECEIVED_COLOR(node.toString())}\n` + `With HTML content:\n  ${jestMatcherUtils.printReceived(nodeHtml)}\n` + `To contain HTML:\n  ${jestMatcherUtils.printExpected(text)}\n`;
  return {
    pass,
    message
  };
}
function printReceivedWithHighlight(text, start, length) {
  return jestMatcherUtils.RECEIVED_COLOR(`"${text.slice(0, start)}${jestMatcherUtils.INVERTED_COLOR(text.slice(start, start + length))}${text.slice(start + length)}"`);
}

exports.toContainReactHtml = toContainReactHtml;
exports.toContainReactText = toContainReactText;
