import type { ComponentType } from 'react';
import type { Node, Predicate, FunctionKeys, DeepPartialArguments, PropsFor, UnknowablePropsFor, DebugOptions, TriggerKeypathParams, TriggerKeypathReturn, KeyPathFunction, ExtractKeypath } from './types';
import { Tag } from './types';
type Root = import('./root').Root<unknown>;
interface Tree<Props> {
    tag: Tag;
    type: string | ComponentType<any> | null;
    props: Props;
    instance?: any;
}
export declare class Element<Props> implements Node<Props> {
    private readonly tree;
    private readonly allChildren;
    readonly root: Root;
    get props(): Props;
    get type(): string | ComponentType<any> | null;
    get isDOM(): boolean;
    get instance(): any;
    get children(): Element<unknown>[];
    private get elementDescendants();
    private get elementChildren();
    get descendants(): Element<unknown>[];
    get domNodes(): HTMLElement[];
    get domNode(): HTMLElement | null;
    private descendantsCache;
    private elementChildrenCache;
    constructor(tree: Tree<Props>, allChildren: (Element<unknown> | string)[], root: Root);
    data(key: string): string | undefined;
    prop<K extends keyof Props>(key: K): Props[K];
    text(): string;
    html(): string;
    is<Type extends ComponentType<any> | string>(type: Type): this is Element<PropsFor<Type>>;
    find<Type extends ComponentType<any> | string>(type: Type, props?: Partial<PropsFor<Type>>): Element<PropsFor<Type>> | null;
    findAll<Type extends ComponentType<any> | string>(type: Type, props?: Partial<PropsFor<Type>>): Element<PropsFor<Type>>[];
    findWhere<Type extends ComponentType<any> | string | unknown = unknown>(predicate: Predicate): Element<UnknowablePropsFor<Type>> | null;
    findAllWhere<Type extends ComponentType<any> | string | unknown = unknown>(predicate: Predicate): Element<UnknowablePropsFor<Type>>[];
    trigger<K extends FunctionKeys<Props>>(prop: K, ...args: DeepPartialArguments<Props[K]>): NonNullable<Props[K]> extends (...args: any[]) => any ? ReturnType<NonNullable<Props[K]>> : never;
    triggerKeypath<Path extends string, ExtractedFunction extends KeyPathFunction = ExtractKeypath<Props, Path>>(...params: TriggerKeypathParams<Props, Path, ExtractedFunction>): TriggerKeypathReturn<Props, Path, ExtractedFunction>;
    debug(options?: DebugOptions): any;
    toString(): any;
}
export {};
//# sourceMappingURL=element.d.ts.map