import React from 'react';
import { Element } from './element';
import type { Node, Predicate, FunctionKeys, DeepPartialArguments, PropsFor, DebugOptions, TriggerKeypathParams, TriggerKeypathReturn, KeyPathFunction, ExtractKeypath } from './types';
type ResolveRoot = (element: Element<unknown>) => Element<unknown> | null;
type Render = (element: React.ReactElement<unknown>) => React.ReactElement<unknown>;
export interface Options {
    render?: Render;
    resolveRoot?: ResolveRoot;
}
export declare const connected: Set<Root<unknown>>;
export declare class Root<Props> implements Node<Props> {
    private tree;
    get props(): Props;
    get isDOM(): boolean;
    get type(): string | React.ComponentType<any> | null;
    get instance(): any;
    get children(): Element<unknown>[];
    get descendants(): Element<unknown>[];
    get domNodes(): HTMLElement[];
    get domNode(): HTMLElement | null;
    private wrapper;
    private element;
    private reactRoot;
    private root;
    private acting;
    private destroyed;
    private actCallbacks;
    private render;
    private resolveRoot;
    private get mounted();
    constructor(tree: React.ReactElement<Props>, { render, resolveRoot }?: Options);
    act<T>(action: () => T, { update }?: {
        update?: boolean | undefined;
    }): T;
    html(): string;
    text(): string;
    is<Type extends React.ComponentType<any> | string>(type: Type): this is Root<PropsFor<Type>>;
    prop<K extends keyof Props>(key: K): Props[K];
    data(key: string): string | undefined;
    find<Type extends React.ComponentType<any> | string>(type: Type, props?: Partial<PropsFor<Type>>): Element<PropsFor<Type>> | null;
    findAll<Type extends React.ComponentType<any> | string>(type: Type, props?: Partial<PropsFor<Type>>): Element<PropsFor<Type>>[];
    findWhere<Type extends React.ComponentType<any> | string | unknown = unknown>(predicate: Predicate): Element<import("./types").UnknowablePropsFor<Type>> | null;
    findAllWhere<Type extends React.ComponentType<any> | string | unknown = unknown>(predicate: Predicate): Element<import("./types").UnknowablePropsFor<Type>>[];
    trigger<K extends FunctionKeys<Props>>(prop: K, ...args: DeepPartialArguments<Props[K]>): NonNullable<Props[K]> extends (...args: any[]) => any ? ReturnType<NonNullable<Props[K]>> : never;
    triggerKeypath<Path extends string, ExtractedFunction extends KeyPathFunction = ExtractKeypath<Props, Path>>(...args: TriggerKeypathParams<Props, Path, ExtractedFunction>): TriggerKeypathReturn<Props, Path, ExtractedFunction>;
    mount(): void;
    unmount(): void;
    destroy(): Promise<void>;
    setProps(props: Partial<Props>): void;
    forceUpdate(): void;
    debug(options?: DebugOptions): any;
    toString(): any;
    private update;
    private ensureRoot;
    private withRoot;
}
export {};
//# sourceMappingURL=root.d.ts.map